# %% bcmfR R-source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# bcmf_bcferr_10.R



# === [SummaryTable_Aqueous] ====================
SummTable_Aqueous <-  function(modelfit  = BCF.ModelFit,
                                  testparam = TEST.Aqueous.Design,
                                  savefit   = TRUE,
                                  digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Aqueous
  summtabaqua <- SummaryTable_Aqueous_GENERIC(modelfit, testparam)
  
  #-- store in global BCFPar.CFish.Joint
  if (savefit)
    assign("BCF.SummTable", summtabaqua, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabaqua
  } else {
    lucid::lucid(summtabaqua, dig = digits)
  }
  
}
# ___ (SummTable_Aqueous) ____________________



# === [SummTable_Aqueous_BoxCox] ====================
SummTable_Aqueous_BoxCox <-  function(modelfit  = BCF.ModelFit.BoxCox,
                                  testparam = TEST.Aqueous.Design,
                                  savefit   = TRUE,
                                  digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Aqueous
  summtabaqua <- SummaryTable_Aqueous_GENERIC(modelfit, testparam)
  
  #-- store in global BCFPar.CFish.Joint
  if (savefit)
    assign("BCF.SummTable.BoxCox", summtabaqua, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabaqua
  } else {
    lucid::lucid(summtabaqua, dig = digits)
  }
  
}
# ___ (SummTable_Aqueous_BoxCox) ____________________



# === [SummTable_Aqueous_Ln] ====================
SummTable_Aqueous_Ln <-  function(modelfit  = BCF.ModelFit.Ln,
                                  testparam = TEST.Aqueous.Design,
                                  savefit   = TRUE,
                                  digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Aqueous
  summtabaqua <- SummaryTable_Aqueous_GENERIC(modelfit, testparam)
  
  #-- store in global BCFPar.CFish.Joint
  if (savefit)
    assign("BCF.SummTable.Ln", summtabaqua, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabaqua
  } else {
    lucid::lucid(summtabaqua, dig = digits)
  }
  
}
# ___ (SummTable_Aqueous_Ln) ____________________



# === [SummaryTable_Aqueous_GENERIC] ====================
# --- FINAL
SummaryTable_Aqueous_GENERIC <- function(modelfit,
                                         testparam = TEST.Aqueous.Design) {

  # get the relevant parameter values from input param vector
  kgrowth       = TEST.Aqueous.Design["kgrowth", "value"]
  lipidfish     = TEST.Aqueous.Design["lipidfish", "value"]
  
  #-- k1
  gfunck1 <- "fitk1"
  
  #-- k2
  gfunck2 <- "fitk2"
  
  #-- k2g elimination rate constant: k2 - kg, a.k.a k2g
  gfunck2g <- paste("(fitk2-", kgrowth, ")", sep = "")
   
  #-- growth-corrected half-life
  gfuncthg <- paste(0.693147, "/(fitk2-", kgrowth,")", sep = "")
  # gfuncthg <- paste(0.693, "/(fitk2-", 0.0373, ")", sep = "")
  
  #-- BCFK = k1/k2
  gfuncbcfk   <- "fitk1/fitk2"

  #-- BCFKg = k1/(k2 - kg)
  gfuncbcfkg  <- paste("fitk1/(fitk2-", kgrowth,")", sep = "")
  # gfuncbcfkg  <- paste("fitk1/(fitk2-", 0.0373,")", sep = "")
  
  #-- BCFKgL lipid corrected
  gfuncbcfkgl <- paste(5, "/", lipidfish, "*fitk1/(fitk2-", kgrowth,")", sep = "")
  # gfuncbcfkgl <- paste(5, "/", 4.65, "*fitk1/(fitk2-", 0.0373,")", sep = "")
  
  
  #-- deltaMethod from package car for k1
  delmethk1     <- car::deltaMethod(modelfit, g = gfunck1)
  
  #-- deltaMethod from package car for k2
  delmethk2     <- car::deltaMethod(modelfit, g = gfunck2)
  
  #-- deltaMethod from package car for k2g
  delmethk2g    <- car::deltaMethod(modelfit, g = gfunck2g)

  #-- deltaMethod from package car for thg (growth-corrected half-life)
  delmeththg    <- car::deltaMethod(modelfit, g = gfuncthg)

  #-- deltaMethod from package car for BCFK
  delmethbcfk   <- car::deltaMethod(modelfit, g = gfuncbcfk)

  #-- deltaMethod from package car for BCFKg
  delmethbcfkg  <- car::deltaMethod(modelfit, g = gfuncbcfkg)

  #-- deltaMethod from package car for BCFKgL
  delmethbcfkgl <- car::deltaMethod(modelfit, g = gfuncbcfkgl)
  
  
  #-- collect individual estimates and standard errors
  estk1     <- delmethk1[[1]]
  estk2     <- delmethk2[[1]]
  estk2g    <- delmethk2g[[1]]
  estthg    <- delmeththg[[1]]
  estbcfk   <- delmethbcfk[[1]]
  estbcfkg  <- delmethbcfkg[[1]]
  estbcfkgl <- delmethbcfkgl[[1]]

  #-- and standard errors
  errk1     <- delmethk1[[2]]
  errk2     <- delmethk2[[2]]
  errk2g    <- delmethk2g[[2]]
  errthg    <- delmeththg[[2]]
  errbcfk   <- delmethbcfk[[2]]
  errbcfkg  <- delmethbcfkg[[2]]
  errbcfkgl <- delmethbcfkgl[[2]]

  #-- build Wald confidence intervals
  clok1     <- estk1 - 1.96*errk1
  cupk1     <- estk1 + 1.96*errk1
   
  clok2     <- estk2 - 1.96*errk2
  cupk2     <- estk2 + 1.96*errk2
  
  clok2g    <- estk2g - 1.96*errk2g
  cupk2g    <- estk2g + 1.96*errk2g

  clothg    <- estthg - 1.96*errthg
  cupthg    <- estthg + 1.96*errthg

  clobcfk   <- estbcfk - 1.96*errbcfk
  cupbcfk   <- estbcfk + 1.96*errbcfk

  clobcfkg  <- estbcfkg - 1.96*errbcfkg
  cupbcfkg  <- estbcfkg + 1.96*errbcfkg

  clobcfkgl <- estbcfkgl - 1.96*errbcfkgl
  cupbcfkgl <- estbcfkgl + 1.96*errbcfkgl
  

  unitnames <- c("L/kgFish/day", "1/day", "1/day", "L/kgFish", "L/kgFish", "day", "L/kgFish")
  
  #-- put the values in a data frame
  dfbcferr <- data.frame(c(estk1, estk2, estk2g, estbcfk, estbcfkg, estthg, estbcfkgl),
                         c(errk1, errk2, errk2g, errbcfk, errbcfkg, errthg, errbcfkgl),
                         c(clok1, clok2, clok2g, clobcfk, clobcfkg, clothg, clobcfkgl),
                         c(cupk1, cupk2, cupk2g, cupbcfk, cupbcfkg, cupthg, cupbcfkgl),
                         unitnames)
  
  #-- set header names
  names(dfbcferr) <- c("Estimate", "Std.Error", "2.5%", "97.5%", "Unit")
  
  #-- set parameter row names
  row.names(dfbcferr) <- c("k1", "k2", "k2g", "BCFK", "BCFKg", "tHalfg", "BCFKgLip")
  
  
  return(dfbcferr)

}
# ___ (SummaryTable_Aqueous_GENERIC) ____________________

# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%