# %% bcmfR 0.3.1 R-Source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# __________________________________________________________________________
#
#    bio_bcfplot_10.R
# __________________________________________________________________________
#
# %% R-code to plot CFish over time for various models

# todo: annotation for roxygen
# todo: option prefix vertical label with transformation, default = TRUE
# todo: set yrange, ylim? Now hardcoded for one example
# todo: change initial blank full curve (type = "n"), or data covering rectangle
#       to set full plot range into something else (set xlim, ylim?)
# todo: line colors
# todo: data symbols, data colors
# todo: ylim transformation finish
plotModel_BCF <- function(time.range = NULL,
                          time.steps = 201,
                          cfish.range = NULL,
                          testparam  = TEST.Aqueous.Param,
                          park1 = NULL,
                          park2 = NULL,
                          transformation  = c("none","boxcox","ln","log10","sqrt"),
                          lambda = NULL,
                          datatoplot = DATA.Aqueous.Work,
                          datacolxy  = c(1,2),
                          maintitle  = "CFish Plot",
                          labelx     = "Time (d)",
                          labely     = "CFish (ugX/kgW)",
                          linetype   = PLOT.SETTING$linetype,
                          # vector of 2 gives linestyle.upt, linestyle.dep
                          linewidth  = PLOT.SETTING$linewidth,
                          plotsymbol = PLOT.SETTING$plotsymbol,
                          plottype   = PLOT.SETTING$plottype) {
  
  cwater <- testparam["cwater", "value"]
  tdepur <- testparam["tdepur", "value"]

  # build the time sequence for plotting
  timeseq     <- seq(time.range[1], time.range[2], length.out = time.steps)
  timeseq.upt <- seq(time.range[1], tdepur,        length.out = floor(time.steps/2) + 1)
  timeseq.dep <- seq(tdepur,        time.range[2], length.out = floor(time.steps/2) + 1)
  
  # if transformation is unspecified, take the first, i.e. "none"
  if (length(transformation) > 1) transformation <- "none"
  
  #-- for transformation == "boxcox", lambda must have a value (not NULL)
  if (tolower(transformation) == "boxcox")
    stopifnot(!is.null(lambda))
  
  # automatically prefix the vertical label with the transformation
  if (tolower(transformation) != "none") {
    if (tolower(transformation) != "boxcox") {
      labely <- paste(transformation, labely, sep = "  ")
    } else {
      labely <- paste(transformation, labely, " Lambda = ", lambda, sep = "  ")
    }
  }
  
  # calc var cfish over time, possibly on a transformed scale
  cfish.scaled <- switch(tolower(transformation),
                         boxcox = car::bcPower(modelRun_Aqueous(timeseq, cwater, tdepur, park1, park2),lambda),
                         none  = modelRun_Aqueous(timeseq, cwater, tdepur, park1, park2),
                         ln    = log(modelRun_Aqueous(timeseq, cwater, tdepur, park1, park2)),
                         log10 = log10(modelRun_Aqueous(timeseq, cwater, tdepur, park1, park2)),
                         sqrt  = sqrt(modelRun_Aqueous(timeseq, cwater, tdepur, park1, park2)),
                         stop("-> Unknown Transformation"))
  
  # calc var cfish over time, possibly on a transformed scale
  cfish.scaled.upt <- switch(tolower(transformation),
                             boxcox = car::bcPower(modelRun_Aqueous(timeseq.upt, cwater, tdepur, park1, park2),lambda),
                             none  = modelRun_Aqueous(timeseq.upt, cwater, tdepur, park1, park2),
                             ln    = log(modelRun_Aqueous(timeseq.upt, cwater, tdepur, park1, park2)),
                             log10 = log10(modelRun_Aqueous(timeseq.upt, cwater, tdepur, park1, park2)),
                             sqrt  = sqrt(modelRun_Aqueous(timeseq.upt, cwater, tdepur, park1, park2)),
                             stop("-> Unknown Transformation"))
  
  cfish.scaled.dep <- switch(tolower(transformation),
                             boxcox = car::bcPower(modelRun_Aqueous(timeseq.dep, cwater, tdepur, park1, park2),lambda),
                             none  = modelRun_Aqueous(timeseq.dep, cwater, tdepur, park1, park2),
                             ln    = log(modelRun_Aqueous(timeseq.dep, cwater, tdepur, park1, park2)),
                             log10 = log10(modelRun_Aqueous(timeseq.dep, cwater, tdepur, park1, park2)),
                             sqrt  = sqrt(modelRun_Aqueous(timeseq.dep, cwater, tdepur, park1, park2)),
                             stop("-> Unknown Transformation"))
  
  #  plot(cfish.scaled ~ timeseq,
  #       lwd  = linewidth,
  #       main = maintitle,
  #       xlab = labelx,
  #       ylab = labely,
  #       type = plottype
  #       )
  
  # ylim.scaled <- switch(transformation,
  #                        none = ylim,
  #                       ln = log(ylim),
  #                       stop("-> Unknown Transformation in ylim.scaled"))
  
  # plot a blank curve to get the frame, scales, ticks, etc. Change later!
  # perhaps first determine data window: min(transformation(model(time.range[1])),min(ydata)),
  #                                      max(max(ydata),transformation(model(tdepur)))
  plot(cfish.scaled ~ timeseq,
       main = maintitle,
       xlab = labelx,
       ylab = labely,
       ylim = cfish.range,
       type = plottype,
       lty  = 0 # blank line
  )
  
  if (length(linetype) == 1) {
    linetype.upt = linetype
    linetype.dep = linetype
  } else {
    linetype.upt = linetype[1]
    linetype.dep = linetype[2]
  }
  
  # plot the curves
  lines(cfish.scaled.upt ~ timeseq.upt,
        #main = maintitle,
        #xlab = labelx,
        #ylab = labely,
        #type = plottype,
        lty  = linetype.upt,
        lwd  = linewidth)
  
  lines(cfish.scaled.dep ~ timeseq.dep,
        #type = plottype,
        lty  = linetype.dep,
        lwd  = linewidth)
  
  
  # plot points when given
  if (!is.null(datatoplot)) {
    datatoplotx <- datatoplot[[datacolxy[1]]]
    datatoploty <- datatoplot[[datacolxy[2]]]
    datatoploty.scaled <- switch(tolower(transformation),
                                 boxcox = car::bcPower(datatoploty, lambda),
                                 none = datatoploty,
                                 ln   = log(datatoploty),
                                 log10 = log10(datatoploty),
                                 sqrt  = sqrt(datatoploty),
                                 stop("-> Unknown Transformation for Data"))
    
    points(datatoplotx,
           datatoploty.scaled,
           pch  = plotsymbol)
  }
}
# ___ plot_Model_BCFK1K2 ____________________


# review for K1K2Ctd?
# quickie to plot model with discontinuity: SEQUENTIAL METHOD 2 with k2 in both phases,
#                                           but intercept of dep phase is free
Plot.CFish.K1GammaK2   <- function(time.range = NULL,
                                   time.steps = 201,
                                   cfish.range = NULL,
                                   
                                   cwater = TEST.Aqueous.Param["cwater"],
                                   tdepur = TEST.Aqueous.Param["tdepur"],
                                    #park1 = PAR.K1START[[1]],
                                    
                                   park1 = NULL,
                                   pargamma = NULL,
                                   park2 = NULL,
                                   
                                   datatoplot = NULL, # formerly: Data.CFish
                                   datacolxy  = c(1,2),
                                   
                                   # "sqrt"?
                                   transformation   = c("none","boxcox","ln","log10"),
                                   lambda = NULL,
                                   yvaluetoadd = 0.,
                                   
                                   maintitle  = "CFish Plot",
                                   labelx     = "Time [d]",
                                   labely     = "CFish [ugX/kgW]",
                                    # vector of 2 gives linestyle.upt, linestyle.dep                          
                                   
                                   linetype   = PLOT.SETTING$linetype,
                                   linewidth  = PLOT.SETTING$linewidth,
                                   plotsymbol = PLOT.SETTING$plotsymbol,
                                   plottype   = PLOT.SETTING$plottype) {
  
  
  #-- time.range is required, or derive from data if present?
  stopifnot(!is.null(time.range)||!is.null(datatoplot))
  
  if(is.null(time.range)&&!is.null(datatoplot))
    time.range <- c(min(datatoplot[datacolxy[1]]), max(datatoplot[datacolxy[1]]))
  
  # build the time sequens for plotting
  timeseq     <- seq(time.range[1], time.range[2], length.out = time.steps)
  timeseq.upt <- seq(time.range[1], tdepur,        length.out = floor(time.steps/2)+1)
  timeseq.dep <- seq(tdepur,        time.range[2], length.out = floor(time.steps/2)+1)
  
  
  # if parameter bcf is given, overrule parameter k1
  # if (!is.null(parbcf)) park1 <- parbcf*park2
  
  # if transformation is unspecified, take the first, i.e. "none"
  if (length(transformation) > 1) transformation <- "none"

  #-- for transformation == "boxcox", lambda must have a value (not NULL)
  if (tolower(transformation) == "boxcox")
    stopifnot(!is.null(lambda))
   
  #-- calc var cfish over time, possibly on a transformed scale
  cfish.scaled <- switch(tolower(transformation),
                         none  = modelCFishK1GammaK2(timeseq, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd,
                         boxcox = car::bcPower(modelCFishK1GammaK2(timeseq, cwater, tdepur, park1, pargamma, park2)
                                          + yvaluetoadd, lambda),
                         ln    = log(modelCFishK1GammaK2(timeseq, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                         log10 = log10(modelCFishK1GammaK2(timeseq, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                         stop("-> Unknown Transformation"))
  
  # calc var cfish over time for timeseq.upt, possibly on a transformed scale
  cfish.scaled.upt <- switch(tolower(transformation),
                             none  = modelCFishK1GammaK2(timeseq.upt, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd,
                             boxcox = car::bcPower(modelCFishK1GammaK2(timeseq.upt, cwater, tdepur, park1, pargamma, park2)
                                     + yvaluetoadd, lambda),
                             ln    = log(modelCFishK1GammaK2(timeseq.upt, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                             log10 = log10(modelCFishK1GammaK2(timeseq.upt, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                             stop("-> Unknown Transformation"))
  
  #-- only for timeseq.dep
  cfish.scaled.dep <- switch(tolower(transformation),
                             none  = modelCFishK1GammaK2(timeseq.dep, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd,
                             boxcox = car::bcPower(modelCFishK1GammaK2(timeseq.dep, cwater, tdepur, park1, pargamma, park2)
                                              + yvaluetoadd, lambda),
                             ln    = log(modelCFishK1GammaK2(timeseq.dep, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                             log10 = log10(modelCFishK1GammaK2(timeseq.dep, cwater, tdepur, park1, pargamma, park2) + yvaluetoadd),
                             stop("-> Unknown Transformation"))

  
  # automatically prefix the vertical label with the transformation
  if (tolower(transformation) != "none") {
    if (tolower(transformation) != "boxcox") {
      labely <- paste(transformation, labely, sep = "  ")
    } else {
      labely <- paste(transformation, labely, "lambda = ", lambda, sep = "  ")
    }
  }
  
  
  #-- plot points when given (no check on NA and zero values)
  if (!is.null(datatoplot)) {
    
    #- horizontal x-data
    datatoplotx <- datatoplot[[datacolxy[1]]]
    
    #- vertical y-data
    datatoploty <- datatoplot[[datacolxy[2]]]
    
    #- transformation vertical y-data as well
    datatoploty.scaled <- switch(tolower(transformation),
                                 none = datatoploty + yvaluetoadd,
                                 boxcox = car::bcPower(datatoploty + yvaluetoadd, lambda),
                                 ln   = log(datatoploty + yvaluetoadd),
                                 log10 = log10(datatoploty + yvaluetoadd),
                                 stop("-> Unknown Transformation for Data"))
    
    #- plot the datapoints invisible to get the frame, scales, ticks, etc.
    #- this ensures (?) that all points are within the plot frame
    plot(datatoplotx,
         datatoploty.scaled,
         main = maintitle,
         xlab = labelx,
         ylab = labely,
         ylim = cfish.range, # check if y-data are out of range??
         col  = "white")

  }

  
  #  plot(cfish.scaled ~ timeseq,
  #       lwd  = linewidth,
  #       main = maintitle,
  #       xlab = labelx,
  #       ylab = labely,
  #       type = plottype
  #       )
  
  # ylim.scaled <- switch(transformation,
  #                        none = ylim,
  #                       ln = log(ylim),
  #                       stop("-> Unknown Transformation in ylim.scaled"))

  #-- assign linetypes to the respective uptake and depuration phases
  if (length(linetype) == 1) {
    linetype.upt = linetype
    linetype.dep = linetype
  } else {
    linetype.upt = linetype[1]
    linetype.dep = linetype[2]
  }
  
  
  
  # plot a blank curve to get the frame, scales, ticks, etc. Change later!
  # perhaps first determine data window: min(transformation(model(time.range[1])),min(ydata)),
  #                                      max(max(ydata),transformation(model(tdepur)))
  #-- if there are no data, we need the curve plot
  if (is.null(datatoplot)) {
    
    #-
    plot(cfish.scaled ~ timeseq,
       main = maintitle,
       xlab = labelx,
       ylab = labely,
       ylim = cfish.range,
       type = plottype,
       col  = "white") # blank line

    #- then add the curves separately
    #- uptake part
    lines(cfish.scaled.upt ~ timeseq.upt,
          #main = maintitle,
          #xlab = labelx,
          #ylab = labely,
          #type = plottype,
          lty  = linetype.upt,
          lwd  = linewidth)
    
    #- depuration part
    lines(cfish.scaled.dep ~ timeseq.dep,
          #type = plottype,
          lty  = linetype.dep,
          lwd  = linewidth)
       
  } else { # but there are data
    
    #- if there are data, then add the curves
    #- uptake part
    lines(cfish.scaled.upt ~ timeseq.upt,
          #main = maintitle,
          #xlab = labelx,
          #ylab = labely,
          #type = plottype,
          lty  = linetype.upt,
          lwd  = linewidth)
    
    #- depuration part
    lines(cfish.scaled.dep ~ timeseq.dep,
          #type = plottype,
          lty  = linetype.dep,
          lwd  = linewidth)
    
    points(datatoplotx,
           datatoploty.scaled)
    
  }

}
#~~ Plot.CFish.K1GammaK2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


# __________________________________________________________________________
# %% END OF R-Source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%