# %% BcfR0.3-1 R-Source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# __________________________________________________________________________
#
#    bcfr0-bcffish-seq2-nogr-5.R
# __________________________________________________________________________
#
# %%

#=== start.Fit.CFish.Seq2.ParK1 ===========================================#
#--- estimates the k1 parameter from Seq1
#-!- model.Start.CFish.Seq2 ??
# start.Fit.CFish.Seq2.ParK1 <- function(datacfishln = Data.CFish.Ln,
#                                  testcwater  = Test.CWater.Value,
#                                  timedepur   = Test.TDepur.Value) {
#   
#   #-- all bcf parameters (k1, k2, bcf) from Fit.BCFPar.CFish.Seq1
#   parfitseq1 <- Fit.BCFPar.CFish.Seq1(datacfishln, testcwater, timedepur)
#   
#   #-- get the k1 parameter, which is the 1st
#   startk1val   <- parfitseq1[["k1"]]
#   
#   #-- make startk1 a list
#   #startk1list <- list(fitk1 = startk1val)
#   
#   #-- return startk1list
#   #startk1list
#   
#   startk1val
# }
#=== start.Fit.CFish.Seq2.ParK1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


# #=== start.Fit.CFish.Seq2.ParBcf ==========================================#
# #--- estimates the k1 parameter from Seq1
# 
# start.Fit.CFish.Seq2.ParBcf <- function(datacfishln = Data.CFish.Ln,
#                                  testcwater  = Test.CWater.Value,
#                                  timedepur   = Test.TDepur.Value) {
#   
#   #-- all bcf parameters (k1, k2, bcf) from Fit.BCFPar.CFish.Seq1
#   parfitseq1 <- Fit.BCFPar.CFish.Seq1(datacfishln, testcwater, timedepur)
#   
#   #-- get the bcf parameter, which is the 3rd
#   startbcfval   <- parfitseq1[["bcf"]]
#   
#   #-- make startbcf a list
#   startbcflist <- list(fitbcf = startbcfval)
#   
#   #-- return start
#   startbcflist
# }
# #=== start.Fit.CFish.Seq2.ParBcf ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== Model.Fit.CFish.Seq2.Uptake ==========================================#
#---

Model.Fit.CFish.Seq2.Uptake <- function(datacfishln = Data.CFish.Ln,
                                        testcwater  = Test.CWater.Value,
                                        timedepur   = Test.TDepur.Value,
                                        startk1     = NULL, # startk1 value
                                        trace       = FALSE,
                                        result      = "object") {
  
  #-- available options
  optionvec <- c("coef","object","options")
  
  #-- return option vector, for result = "options"
  if (tolower(result) == "options") return(optionvec)
  
  #-- get all BcfPar stats for Seq1: k1, gamma, k2, bcf
  statsbcfparseq1 <- stats.BCFPar.CFish.Seq1(datacfishln = datacfishln,
                                             testcwater  = testcwater,
                                             timedepur   = timedepur)
  
  #-- k2hat is the third in column Estimate (how to do this better?)
  k2hat <- statsbcfparseq1$Estimate[3]
  
  #-- if startk1 == NULL then get startk1 from stats.BCFPar.CFish.Seq1 also
  if (is.null(startk1)) {
    startk1 <- statsbcfparseq1$Estimate[1]
  } #-- endif
  
  #-- nls() requires start to be a list
  startk1list <- list(fitk1 = startk1)
  
  #-- nonlinear model fit on ln-transformed data for k2-k2 params
  modlncfishseq2uptake <- nls(var.lncfish ~
                            log(Model.Run.CFish.K1K2.Uptake(var.time,
                                                            testcwater,
                                                            fitk1, k2hat)),
                              data  = Data.CFish.Ln.Uptake,
                              start = startk1list, 
                              trace = trace)
  
  #-- return the model object
  modlncfishseq2uptake
  
}
#=== Model.Fit.CFish.Seq2.Uptake ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#


# #=== Model.Fit.CFish.Seq2.ParK1 ===========================================#
# #---
# 
# Model.Fit.CFish.Seq2.ParK1 <- function(datacfishln = Data.CFish.Ln,
#                                  testcwater  = Test.CWater.Value,
#                                  timedepur   = Test.TDepur.Value,
#                                  startk1     = NULL, # startk1 value
#                                  trace       = FALSE,
#                                  result      = "object") {
#   
#   #-- available options
#   optionvec <- c("coef","object","options")
#   
#   #-- return option vector, for result = "options"
#   if (tolower(result) == "options") return(optionvec)
#   
#   #-- k2 statistics are calculated from bcfr0-bcffish-seq1-nogr-**.R
#   #-- this ensures Data.CFish.Ln.Depur and Data.CFish.Ln.Uptake to exist
#   statstablek2 <- stats.BCFPar.CFish.Seq1.ParK2(datacfishln = datacfishln,
#                                       timedepur   = timedepur)
#   #-- get k2 point estimate k2hat (and k2err?)
#   k2hat <- statstablek2$Estimate
#   #?? k2err <- statstablek2$Std.Error
#   
#   #-- if startk1 == NULL then get startk1 from Fit.BCFPar.CFish.Seq1
#   if (is.null(startk1)) {
#     startk1 <- start.Fit.CFish.Seq2.ParK1(datacfishln, testcwater, timedepur)
#   } #-- endif
#   
#   #-- nls() requires start to be a list
#   startk1list <- list(fitk1 = startk1)
#   
#   #-- nonlinear model fit on ln-transformed data for k2-k2 params
#   modlncfishseq2_k1 <- nls(var.lncfish ~
#                            log(Model.Run.CFish.K1K2.Uptake(var.time,
#                                                            testcwater,
#                                                            fitk1, k2hat)),
#                            data  = Data.CFish.Ln.Uptake,
#                            start = startk1list, 
#                            trace = trace)
#   
#   #-- return the model object
#   modlncfishseq2_k1
#  
# }
# #=== Model.Fit.CFish.Seq2.ParK1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== Model.Fit.CFish.Seq2.ParBcf ==========================================#
#---
# Model.Fit.CFish.Seq2.ParBcf <- function(datacfishln = Data.CFish.Ln,
#                                        testcwater  = Test.CWater.Value,
#                                        timedepur   = Test.TDepur.Value,
#                                        start       = NULL,
#                                        trace       = FALSE,
#                                        result      = "object") {
#   
#   #-- available options
#   optionvec <- c("coef","object","options")
#   
#   #-- return option vector, for result = "options"
#   if (tolower(result) == "options") return(optionvec)
#   
#   #-- k2 statistics are calculated from bcfr0-bcffish-seq1-nogr-**.R
#   #-- this ensures Data.CFish.Ln.Depur and Data.CFish.Ln.Uptake to exist
#   statstablek2 <- stats.BCFPar.CFish.ParK2(datacfishln = datacfishln,
#                                       timedepur   = timedepur)
#   #-- get k2hat and k2err
#   k2hat <- statstablek2$Estimate
#   k2err <- statstablek2$Std.Error
#   
#   #-- if start == NULL then get startbcf from Fit.BCFPar.CFish.Seq1
#   if (is.null(start)) {
#     startbcflist <- start.Fit.CFish.Seq2.ParBcf(datacfishln, testcwater,
#                                                 timedepur)
#   } #-- endif
#   
#   #-- nonlinear model fit on ln-transformed data for k2-k2 params
#   modlncfishseq2_bcf <- nls(Var.LnCFish ~
#                              log(Model.Run.CFish.BcfK2.Uptake(Var.Time,
#                                                            testcwater,
#                                                            fitbcf, k2hat)),
#                            data  = Data.CFish.Ln.Uptake,
#                            start = startbcflist, 
#                            trace = trace)
#   
#   #-- return the model object
#   modlncfishseq2_bcf
#   
# }
# #=== Model.Fit.CFish.Seq2.ParBcf ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== stats.BCFPar.CFish.Seq2 ==============================================#
#---
stats.BCFPar.CFish.Seq2 <- function(datacfishln = Data.CFish.Ln,
                                    testcwater  = Test.CWater.Value,
                                    timedepur   = Test.TDepur.Value,
                                    digits      = NULL) {
  
  #-- get stats of k1, gamma, k2, and bcf for Seq1
  statsbcfparseq1 <- stats.BCFPar.CFish.Seq1(datacfishln = datacfishln,
                                             testcwater  = testcwater,
                                             timedepur   = timedepur)
  #== par gamma ==#
  #-- extract gamma stats
  estgamma    <- statsbcfparseq1["gamma","Estimate"]
  errgamma    <- statsbcfparseq1["gamma","Std.Error"] # no space here
  conflogamma <- statsbcfparseq1["gamma","2.5%"]
  confupgamma <- statsbcfparseq1["gamma","97.5%"]
  
  #== par k2 ==#
  #-- extract k2 stats
  estk2       <- statsbcfparseq1["k2","Estimate"]
  errk2       <- statsbcfparseq1["k2","Std.Error"] # no space here
  conflok2    <- statsbcfparseq1["k2","2.5%"]
  confupk2    <- statsbcfparseq1["k2","97.5%"]
    
  #== par k1 ==#
  #-- nls fit of k1 to Uptake phase with given k2
  modseq2uptake <- Model.Fit.CFish.Seq2.Uptake(datacfishln,
                                               testcwater, timedepur)
  
  #-- coef summary: Estimate, Std. Error (space!), t value, Pr(>|t|)
  #-- is of class "matrix"
  coefsumm <- coef(summary(modseq2uptake))

  #-- estimate and std.error
  estk1 <- coefsumm["fitk1", "Estimate"]
  errk1 <- coefsumm["fitk1", "Std. Error"]

  #-- build Wald confidence intervals
  conflok1 <- estk1 - 1.96*errk1
  confupk1 <- estk1 + 1.96*errk1

  #== bcf ==#
  #-- bcf is ratio of k1 and k2
  estbcf <- estk1/estk2

  #-- bcf err comes from deltamethod in Mood & Graybill
  errbcf <- stderr.BCF.K1K2(k1hat = estk1,
                            k2hat = estk2,
                            k1stderr = errk1,
                            k2stderr = errk2) 
                            # assume k1k2cov = 0. (default)
  
  #-- bcf conf limits
  conflobcf <- estbcf - 1.96*errbcf
  confupbcf <- estbcf + 1.96*errbcf
  
  #-- put the values in a data frame
  dfk1gammak2bcf <- data.frame(c(estk1, estgamma,estk2, estbcf),
                               c(errk1, errgamma,errk2, errbcf),
                               c(conflok1, conflogamma,conflok2, conflobcf),
                               c(confupk1, confupgamma,confupk2, confupbcf))
  
  #-- set header names
  names(dfk1gammak2bcf) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
  #-- set parameter row names
  row.names(dfk1gammak2bcf) <- c("k1", "gamma","k2", "bcf")
  
  #-- store in global BCFPar.CFish.Seq2
  BCFPar.CFish.Seq2 <<- dfk1gammak2bcf
  
  #-- return dfk1gammak2bcf
  if(is.null(digits)) {
    dfk1gammak2bcf
  } else {
    lucid(dfk1gammak2bcf, dig=digits)
  }
  
}
#=== stats.BCFPar.CFish.Seq2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== fit.BCFPar.CFish.Seq2 ================================================#
#--- bcf parameters: k1, k2, bcf as point estimates for sequential #2

fit.BCFPar.CFish.Seq2 <- function (datacfishln = Data.CFish.Ln,
                                   testcwater = Test.CWater.Value,
                                   timedepur  = Test.TDepur.Value) {
  #-- default method = "delta".
  
  #-- calc full summary table of parameter statss
  statsk1k2bcfdataframe <- stats.BCFPar.CFish.Seq2(datacfishln, testcwater,
                                                   timedepur)
  
  #-- take the vector of point estimates: first column
  park1k2bcfestvec <- statsk1k2bcfdataframe[[1]]
  
  #-- attach names to the elements
  names(park1k2bcfestvec) <- c("k1", "gamma", "k2", "bcf")
  
  #-- return the named point estimate vector
  park1k2bcfestvec
  
}
#=== fit.BCFPar.CFish.Seq2 ________________________________________________.

# __________________________________________________________________________
# %% END OF R-SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%